@@ -112,6 +112,7 @@ class ELFStructs(object):
         self._create_riscv_attributes()
         self._create_elf_hash()
         self._create_gnu_hash()
+        self._create_gnu_debuglink()
 
     #-------------------------------- PRIVATE --------------------------------#
 
@@ -189,6 +190,8 @@ class ELFStructs(object):
         sh_type_dict = ENUM_SH_TYPE_BASE
         if self.e_machine == 'EM_ARM':
             sh_type_dict = ENUM_SH_TYPE_ARM
+        elif self.e_machine == 'EM_AARCH64':
+            sh_type_dict = ENUM_SH_TYPE_AARCH64           
         elif self.e_machine == 'EM_X86_64':
             sh_type_dict = ENUM_SH_TYPE_AMD64
         elif self.e_machine == 'EM_MIPS':
@@ -404,6 +407,10 @@ class ELFStructs(object):
                 return None
             if ctx.pr_type.startswith('GNU_PROPERTY_X86_'):
                 return ('GNU_PROPERTY_X86_*', 4, 0)
+            elif ctx.pr_type.startswith('GNU_PROPERTY_AARCH64_'):
+                return ('GNU_PROPERTY_AARCH64_*', 4, 0)
+            elif ctx.pr_type.startswith('GNU_PROPERTY_RISCV_'):
+                return ('GNU_PROPERTY_RISCV_*', 4, 0)
             return (ctx.pr_type, ctx.pr_datasz, self.elfclass)
 
         self.Elf_Prop = Struct('Elf_Prop',
@@ -413,6 +420,8 @@ class ELFStructs(object):
                     ('GNU_PROPERTY_STACK_SIZE', 4, 32): self.Elf_word('pr_data'),
                     ('GNU_PROPERTY_STACK_SIZE', 8, 64): self.Elf_word64('pr_data'),
                     ('GNU_PROPERTY_X86_*', 4, 0): self.Elf_word('pr_data'),
+                    ('GNU_PROPERTY_AARCH64_*', 4, 0): self.Elf_word('pr_data'),
+                    ('GNU_PROPERTY_RISCV_*', 4, 0): self.Elf_word('pr_data'),
                 },
                 default=Field('pr_data', lambda ctx: ctx.pr_datasz)
             ),
@@ -554,3 +563,9 @@ class ELFStructs(object):
                                self.Elf_word('bloom_shift'),
                                Array(lambda ctx: ctx['bloom_size'], self.Elf_xword('bloom')),
                                Array(lambda ctx: ctx['nbuckets'], self.Elf_word('buckets')))
+        
+    def _create_gnu_debuglink(self):
+        self.Gnu_debuglink = Struct('Gnu_debuglink',
+            CString("filename"),
+            Padding(lambda ctx: 3 - len(ctx.filename) % 4, strict=True),
+            self.Elf_word("checksum"))
