@@ -14,7 +14,7 @@
 # This code is in the public domain
 #-------------------------------------------------------------------------------
 import argparse
-import os, sys, posixpath
+import os, sys
 import traceback
 
 # For running from development directory. It should take precedence over the
@@ -23,7 +23,7 @@ sys.path.insert(0, '.')
 
 from elftools import __version__
 from elftools.common.exceptions import DWARFError, ELFError
-from elftools.common.py3compat import bytes2str
+from elftools.common.utils import bytes2str
 from elftools.elf.elffile import ELFFile
 from elftools.dwarf.locationlists import LocationParser, LocationEntry, LocationExpr, LocationViewPair, BaseAddressEntry as LocBaseAddressEntry
 from elftools.dwarf.ranges import RangeEntry # ranges.BaseAddressEntry collides with the one above
@@ -84,6 +84,12 @@ def _safe_DIE_linkage_name(die, default=None):
 def _desc_ref(attr, die, extra=''):
     if extra:
         extra = " \"%s\"" % extra
+    if attr.form == 'DW_FORM_ref_addr':
+        return "0x%016x%s" % (attr.value, extra)
+    if attr.form == 'DW_FORM_ref_sig8':
+        return "0x%016x" % attr.value
+    # TODO: leading zeros on the addend to CU - sometimes present, sometimes not.
+    # Check by the LLVM sources.    
     return "cu + 0x%04x => {0x%08x}%s" % (
         attr.raw_value,
         die.cu.cu_offset + attr.raw_value,
@@ -99,7 +105,7 @@ def _desc_strx(attr, die):
     return "indexed (%08x) string = \"%s\"" % (attr.raw_value, bytes2str(attr.value).replace("\\", "\\\\"))
 
 FORM_DESCRIPTIONS = dict(
-    DW_FORM_string=lambda attr, die: "\"%s\"" % (bytes2str(attr.value),),
+    DW_FORM_string=lambda attr, die: "\"%s\"" % (bytes2str(attr.value).replace("\\", "\\\\"),),
     DW_FORM_strp=lambda attr, die: " .debug_str[0x%08x] = \"%s\"" % (attr.raw_value, bytes2str(attr.value).replace("\\", "\\\\")),
     DW_FORM_strx1=_desc_strx,
     DW_FORM_strx2=_desc_strx,
@@ -147,13 +153,13 @@ def _desc_decl_file(attr, die):
         if dir_index >= 0:
             dir = bytes2str(includes[dir_index])
             if dir.startswith('.'):
-                dir = posixpath.join(_cu_comp_dir(cu), dir)
+                dir = os.path.join(_cu_comp_dir(cu), dir)
         else:
             dir = _cu_comp_dir(cu)
         file_name = bytes2str(file_entry.name)
     else:
         raise DWARFError("Invalid source filename entry index in a decl_file attribute")
-    return "\"%s\"" % (posixpath.join(dir, file_name),)
+    return "\"%s\"" % (os.path.join(dir, file_name),)
 
 
 def _desc_ranges(attr, die):
@@ -223,7 +229,8 @@ def _arch(cu):
     return cu.dwarfinfo.config.machine_arch
 
 def _desc_reg(reg_no, cu):
-    return describe_reg_name(reg_no, _arch(cu), True).upper()
+    reg_name = describe_reg_name(reg_no, _arch(cu), False)
+    return reg_name.upper() if reg_name else ""
 
 def _desc_operation(op, op_name, args, cu):
     # Not sure about regx(regno) and bregx(regno, offset)
@@ -325,7 +332,9 @@ ATTR_DESCRIPTIONS = dict(
     DW_AT_call_line=_desc_value,
     DW_AT_call_file=_desc_decl_file,
     DW_AT_abstract_origin=_desc_origin,
-    DW_AT_specification=_desc_spec
+    DW_AT_specification=_desc_spec,
+    DW_AT_call_site_value=lambda attr, die: _desc_expression(attr.value, die) if attr.form.startswith('DW_FORM_block') else _desc_locations(attr, die),
+    DW_AT_GNU_call_site_value=lambda attr, die: _desc_expression(attr.value, die) if attr.form.startswith('DW_FORM_block') else _desc_locations(attr, die),
 )
 
 class ReadElf(object):
@@ -342,7 +351,7 @@ class ReadElf(object):
         self.elffile = ELFFile(file)
         self.output = output
         self._dwarfinfo = self.elffile.get_dwarf_info()
-        arches = {"EM_386": "i386", "EM_X86_64": "x86-64"}
+        arches = {"EM_386": "i386", "EM_X86_64": "x86-64", "EM_ARM": "littlearm", "EM_AARCH64": "littleaarch64", "EM_LOONGARCH": "loongarch", "EM_RISCV": "littleriscv", "EM_MIPS": "mips", "EM_TI_C2000": "unknown"}
         arch = arches[self.elffile['e_machine']]
         bits = self.elffile.elfclass
         self._emitline("%s:	file format elf%d-%s" % (filename, bits, arch))
@@ -362,8 +371,7 @@ class ReadElf(object):
         self._emitline(".debug_info contents:")
         for cu in self._dwarfinfo.iter_CUs():
             if cu.header.version >= 5:
-                ut = next(k for (k,v) in ENUM_DW_UT.items() if v == cu.header.unit_type)
-                unit_type_str = " unit_type = %s," % ut
+                unit_type_str = " unit_type = %s," % cu.header.unit_type
             else:
                 unit_type_str = ''
 
@@ -384,13 +392,16 @@ class ReadElf(object):
                 if not die.is_null(): 
                     self._emitline("0x%08x: %s [%d] %s %s" % (
                         die.offset,
-                        die.tag,
+                        die.tag if isinstance(die.tag, str) else "DW_TAG_unknown_%x" % die.tag,
                         die.abbrev_code,
                         '*' if die.has_children else '',
                         '(0x%08x)' % die.get_parent().offset if die.get_parent() is not None else ''))
                     for attr_name in die.attributes:
                         attr = die.attributes[attr_name]
-                        self._emitline("              %s [%s]	(%s)" % (attr_name, attr.form, self.describe_attr_value(die, attr)))
+                        self._emitline("              %s [%s]	(%s)" % (
+                            attr_name if isinstance(attr_name, str) else "DW_AT_unknown_%x" % (attr_name,),
+                            attr.form,
+                            self.describe_attr_value(die, attr)))
                 else:
                     self._emitline("0x%08x: NULL" % (die.offset,))
                     parent = die.get_parent()
@@ -534,6 +545,7 @@ def main(stream=None):
     del ENUM_DW_TAG["DW_TAG_template_value_param"]
     ENUM_DW_TAG['DW_TAG_template_type_parameter'] = 0x2f
     ENUM_DW_TAG['DW_TAG_template_value_parameter'] = 0x30
+    del ENUM_DW_TAG['DW_TAG_lo_user'] # dwarfdump dumps it as unknown
 
     with open(args.file, 'rb') as file:
         try:
