@@ -487,6 +487,7 @@ class ReadElf(object):
                 symbol_name = symbol.name
                 # Print section names for STT_SECTION symbols as readelf does
                 if (symbol['st_info']['type'] == 'STT_SECTION'
+                    and symbol['st_shndx'] != 'SHN_UNDEF'
                     and symbol['st_shndx'] < self.elffile.num_sections()
                     and symbol['st_name'] == 0):
                     symbol_name = self.elffile.get_section(symbol['st_shndx']).name
@@ -549,6 +550,8 @@ class ReadElf(object):
                 elif tag.entry.d_tag in ('DT_MIPS_SYMTABNO',
                                          'DT_MIPS_LOCAL_GOTNO'):
                     parsed = str(tag.entry.d_val)
+                elif tag.entry.d_tag == 'DT_AARCH64_BTI_PLT':
+                    parsed = ''
                 else:
                     parsed = '%#x' % tag['d_val']
 
@@ -573,7 +576,7 @@ class ReadElf(object):
                       self._emitline('  %s %s\t%s' % (
                           note['n_name'].ljust(20),
                           self._format_hex(note['n_descsz'], fieldsize=8),
-                          describe_note(note)))
+                          describe_note(note, self.elffile.header.e_machine)))
 
     def display_relocations(self):
         """ Display the relocations contained in the file
@@ -912,6 +915,7 @@ class ReadElf(object):
 
         if dump_what == 'info':
             self._dump_debug_info()
+            self._dump_debug_types()
         elif dump_what == 'decodedline':
             self._dump_debug_line_programs()
         elif dump_what == 'frames':
@@ -1193,6 +1197,58 @@ class ReadElf(object):
 
         self._emitline()
 
+    def _dump_debug_types(self):
+        """Dump the debug types section
+        """
+        if not self._dwarfinfo.has_debug_info:
+            return
+        if self._dwarfinfo.debug_types_sec is None:
+            return
+        self._emitline('Contents of the %s section:\n' % self._dwarfinfo.debug_types_sec.name)
+
+        # Offset of the .debug_types section in the stream
+        section_offset = self._dwarfinfo.debug_types_sec.global_offset
+
+        for tu in self._dwarfinfo.iter_TUs():
+            self._emitline('  Compilation Unit @ offset %s:' %
+                           self._format_hex(tu.tu_offset, alternate=True))
+            self._emitline('   Length:        %s (%s)' % (self._format_hex(tu['unit_length']),
+                                                          '%s-bit' % tu.dwarf_format()))
+            self._emitline('   Version:       %s' % tu['version'])
+            self._emitline('   Abbrev Offset: %s' % (self._format_hex(tu['debug_abbrev_offset'], alternate=True)))
+            self._emitline('   Pointer Size:  %s' % tu['address_size'])
+            self._emitline('   Signature:     0x%x' % tu['signature'])
+            self._emitline('   Type Offset:   0x%x' % tu['type_offset'])
+
+            die_depth = 0
+            for die in tu.iter_DIEs():
+                self._emitline(' <%s><%x>: Abbrev Number: %s%s' % (
+                    die_depth,
+                    die.offset,
+                    die.abbrev_code,
+                    (' (%s)' % die.tag) if not die.is_null() else ''))
+                if die.is_null():
+                    die_depth -= 1
+                    continue
+
+                for attr in die.attributes.values():
+                    name = attr.name
+                    # Unknown attribute values are passed-through as integers
+                    if isinstance(name, int):
+                        name = 'Unknown AT value: %x' % name
+
+                    attr_desc = describe_attr_value(attr, die, section_offset)
+
+                    self._emitline('    <%x>   %-18s: %s' % (
+                        attr.offset,
+                        name,
+                        attr_desc))
+
+                if die.has_children:
+                    die_depth += 1
+
+        self._emitline()
+
     def _dump_debug_line_programs(self):
         """ Dump the (decoded) line programs from .debug_line
             The programs are dumped in the order of the CUs they belong to.
@@ -1207,7 +1263,7 @@ class ReadElf(object):
             # Avoid dumping same lineprogram multiple times
             lineprogram = self._dwarfinfo.line_program_for_CU(cu)
 
-            if lineprogram in lineprogram_list:
+            if lineprogram is None or lineprogram in lineprogram_list:
                 continue 
 
             lineprogram_list.append(lineprogram)
@@ -1280,6 +1336,9 @@ class ReadElf(object):
                     self._format_hex(entry['CIE_id'], fieldsize=8, lead0x=False)))
                 self._emitline('  Version:               %d' % entry['version'])
                 self._emitline('  Augmentation:          "%s"' % bytes2str(entry['augmentation']))
+                if(entry['version'] >= 4):
+                    self._emitline('  Pointer Size:          %d' % entry['address_size'])
+                    self._emitline('  Segment Size:          %d' % entry['segment_size'])
                 self._emitline('  Code alignment factor: %u' % entry['code_alignment_factor'])
                 self._emitline('  Data alignment factor: %d' % entry['data_alignment_factor'])
                 self._emitline('  Return address column: %d' % entry['return_address_register'])
@@ -1291,9 +1350,11 @@ class ReadElf(object):
                 self._emitline()
 
             elif isinstance(entry, FDE):
+                # Readelf bug #31973
+                length = entry['length'] if entry.cie.offset < entry.offset else entry.cie['length']
                 self._emitline('\n%08x %s %s FDE cie=%08x pc=%s..%s' % (
                     entry.offset,
-                    self._format_hex(entry['length'], fullhex=True, lead0x=False),
+                    self._format_hex(length, fullhex=True, lead0x=False),
                     self._format_hex(entry['CIE_pointer'], fieldsize=8, lead0x=False),
                     entry.cie.offset,
                     self._format_hex(entry['initial_location'], fullhex=True, lead0x=False),
@@ -1426,9 +1487,11 @@ class ReadElf(object):
                 ra_regnum = entry['return_address_register']
 
             elif isinstance(entry, FDE):
+                # Readelf bug #31973 - FDE length misreported if FDE precedes its CIE
+                length = entry['length'] if entry.cie.offset < entry.offset else entry.cie['length']
                 self._emitline('\n%08x %s %s FDE cie=%08x pc=%s..%s' % (
                     entry.offset,
-                    self._format_hex(entry['length'], fullhex=True, lead0x=False),
+                    self._format_hex(length, fullhex=True, lead0x=False),
                     self._format_hex(entry['CIE_pointer'], fieldsize=8, lead0x=False),
                     entry.cie.offset,
                     self._format_hex(entry['initial_location'], fullhex=True, lead0x=False),
@@ -1550,10 +1613,31 @@ class ReadElf(object):
             # Present but empty locations section - readelf outputs a message
             self._emitline("\nSection '%s' has no debugging data." % (section_name,))
             return
+        
+        # The v5 loclists section consists of CUs - small header, and 
+        # an optional loclist offset table. Readelf prints out the header data dump
+        # on top of every CU, so we should too. But we don't scroll through
+        # the loclists section, we are effectively scrolling through the info
+        # section (as to not stumble upon gaps in the secion, and not miss the
+        # GNU locviews, so we have to keep track which loclists-CU we are in.
+        # Same logic in v5 rnglists section dump.
+        lcus = list(loc_lists_sec.iter_CUs()) if ver5 else None
+        lcu_index = 0
+        next_lcu_offset = 0
 
         self._emitline('Contents of the %s section:\n' % (section_name,))
-        self._emitline('    Offset   Begin            End              Expression')
+        if not ver5:
+            self._emitline('    Offset   Begin            End              Expression')
+
         for loc_list in loc_lists:
+            # Emit CU headers before the current loclist, if we've moved to the next CU
+            if ver5 and loc_list[0].entry_offset > next_lcu_offset:
+                while loc_list[0].entry_offset > next_lcu_offset:
+                    lcu = lcus[lcu_index]
+                    self._dump_debug_loclists_CU_header(lcu)
+                    next_lcu_offset = lcu.offset_after_length + lcu.unit_length
+                    lcu_index += 1
+                self._emitline('    Offset   Begin            End              Expression')
             self._dump_loclist(loc_list, line_template, cu_map)
 
     def _dump_loclist(self, loc_list, line_template, cu_map):
@@ -1618,6 +1702,19 @@ class ReadElf(object):
         last = loc_list[-1]
         self._emitline("    %08x <End of list>" % (last.entry_offset + last.entry_length))
 
+    def _dump_debug_loclists_CU_header(self, cu):
+        # Header slightly different from that of v5 rangelist in-section CU header dump
+        self._emitline('Table at Offset %s' % self._format_hex(cu.cu_offset, alternate=True))
+        self._emitline('  Length:          %s' % self._format_hex(cu.unit_length, alternate=True))
+        self._emitline('  DWARF version:   %d' % cu.version)
+        self._emitline('  Address size:    %d' % cu.address_size)
+        self._emitline('  Segment size:    %d' % cu.segment_selector_size)
+        self._emitline('  Offset entries:  %d\n' % cu.offset_count)
+        if cu.offsets and len(cu.offsets):
+            self._emitline('  Offsets starting at 0x%x:' % cu.offset_table_offset)
+            for i_offset in enumerate(cu.offsets):
+                self._emitline('    [%6d] 0x%x' % i_offset)        
+
     def _dump_debug_ranges(self):
         # TODO: GNU readelf format doesn't need entry_length?
         di = self._dwarfinfo
@@ -1631,6 +1728,18 @@ class ReadElf(object):
         else:
             self._dump_debug_rangesection(di, range_lists_sec)
 
+    def _dump_debug_rnglists_CU_header(self, cu):
+        self._emitline(' Table at Offset: %s:' % self._format_hex(cu.cu_offset, alternate=True))
+        self._emitline('  Length:          %s' % self._format_hex(cu.unit_length, alternate=True))
+        self._emitline('  DWARF version:   %d' % cu.version)
+        self._emitline('  Address size:    %d' % cu.address_size)
+        self._emitline('  Segment size:    %d' % cu.segment_selector_size)
+        self._emitline('  Offset entries:  %d\n' % cu.offset_count)
+        if cu.offsets and len(cu.offsets):
+            self._emitline('  Offsets starting at 0x%x:' % cu.offset_table_offset)
+            for i_offset in enumerate(cu.offsets):
+                self._emitline('    [%6d] 0x%x' % i_offset)
+
     def _dump_debug_rangesection(self, di, range_lists_sec):
         # Last amended to match readelf 2.41
         ver5 = range_lists_sec.version >= 5
@@ -1639,53 +1748,48 @@ class ReadElf(object):
         addr_width = addr_size * 2 # In hex digits, 8 or 16
         line_template = "    %%08x %%0%dx %%0%dx %%s" % (addr_width, addr_width)
         base_template = "    %%08x %%0%dx (base address)" % (addr_width)
+        base_template_indexed = "    %%08x %%0%dx (base address index) %%0%dx (base address)" % (addr_width, addr_width)
 
         # In order to determine the base address of the range
         # We need to know the corresponding CU.
         cu_map = {die.attributes['DW_AT_ranges'].value : cu  # Range list offset => CU
             for cu in di.iter_CUs()
             for die in cu.iter_DIEs()
-            if 'DW_AT_ranges' in die.attributes}        
+            if 'DW_AT_ranges' in die.attributes}
+        
+        rcus = list(range_lists_sec.iter_CUs()) if ver5 else None
+        rcu_index = 0
+        next_rcu_offset = 0
 
-        if ver5: # Dump by CUs - unsure at this point what does readelf do, ranges dump is buggy in 2.41
-            self._emitline('Contents of the %s section:\n\n\n' % section_name)
-            for cu in range_lists_sec.iter_CUs():
-                self._emitline(' Table at Offset: %s:' % self._format_hex(cu.cu_offset, alternate=True))
-                self._emitline('  Length:          %s' % self._format_hex(cu.unit_length, alternate=True))
-                self._emitline('  DWARF version:   %d' % cu.version)
-                self._emitline('  Address size:    %d' % cu.address_size)
-                self._emitline('  Segment size:    %d' % cu.segment_selector_size)
-                self._emitline('  Offset entries:  %d\n' % cu.offset_count)
-                # Is the offset table dumped too?
-                for (i, range_list) in enumerate(range_lists_sec.iter_CU_range_lists_ex(cu)):
-                    list_offset = range_list[0].entry_offset
-                    range_list = list(range_lists_sec.translate_v5_entry(entry, cu_map[list_offset]) for entry in range_list)
-                    self._emitline('  Offset: %s, Index: %d' % (self._format_hex(list_offset, alternate=True), i))
-                    self._emitline('    Offset   Begin    End')
-                    self._dump_rangelist(range_list, cu_map, ver5, line_template, base_template)
-        else: # Dump by DIE reference offset
-            range_lists = list(range_lists_sec.iter_range_lists())
-            if len(range_lists) == 0:
-                # Present but empty ranges section - readelf outputs a message
-                self._emitline("\nSection '%s' has no debugging data." % section_name)
-                return
+        range_lists = list(range_lists_sec.iter_range_lists())
+        if len(range_lists) == 0:
+            # Present but empty ranges section - readelf outputs a message
+            self._emitline("\nSection '%s' has no debugging data." % section_name)
+            return
 
-            self._emitline('Contents of the %s section:\n\n\n' % section_name)
+        self._emitline('Contents of the %s section:\n\n\n' % section_name)
+        if not ver5:
             self._emitline('    Offset   Begin    End')
 
-            for range_list in range_lists:
-                if len(range_list) == 0: # working around a bogus behavior in readelf 2.41
-                    # No entries means no offset. Dirty hack: peek the stream position
-                    range_list_offset = range_lists_sec.stream.tell() - self._dwarfinfo.config.default_address_size*2
-                    self._emitline('    %08x <End of list>' % (range_list_offset))
-                else:
-                    self._dump_rangelist(range_list, cu_map, ver5, line_template, base_template)
+        for range_list in range_lists:
+            # Emit CU headers before the curernt rangelist
+            if ver5 and range_list[0].entry_offset > next_rcu_offset:
+                while range_list[0].entry_offset > next_rcu_offset:
+                    rcu = rcus[rcu_index]
+                    self._dump_debug_rnglists_CU_header(rcu)
+                    next_rcu_offset = rcu.offset_after_length + rcu.unit_length
+                    rcu_index += 1
+                self._emitline('    Offset   Begin    End')
+            self._dump_rangelist(range_list, cu_map, ver5, line_template, base_template, base_template_indexed, range_lists_sec)
 
-    def _dump_rangelist(self, range_list, cu_map, ver5, line_template, base_template):
+        # TODO: trailing empty CUs, if any?
+
+    def _dump_rangelist(self, range_list, cu_map, ver5, line_template, base_template, base_template_indexed, range_lists_sec):
         # Weird discrepancy in binutils: for DWARFv5 it outputs entry offset,
         # for DWARF<=4 list offset.
         first = range_list[0]
         base_ip = _get_cu_base(cu_map[first.entry_offset])
+        raw_v5_rangelist = None
         for entry in range_list:
             if isinstance(entry, RangeEntry):
                 postfix = ' (start == end)' if entry.begin_offset == entry.end_offset else ''
@@ -1696,9 +1800,22 @@ class ReadElf(object):
                     postfix))
             elif isinstance(entry,RangeBaseAddressEntry):
                 base_ip = entry.base_address
-                self._emitline(base_template % (
-                    entry.entry_offset if ver5 else first.entry_offset,
-                    entry.base_address))
+                # V5 base entries with index are reported differently in readelf - need to go back to the raw V5 format
+                # Maybe other subtypes too, but no such cases  in the test corpus
+                raw_v5_entry = None
+                if ver5:
+                    if not raw_v5_rangelist:
+                        raw_v5_rangelist = range_lists_sec.get_range_list_at_offset_ex(range_list[0].entry_offset)
+                    raw_v5_entry = next(re for re in raw_v5_rangelist if re.entry_offset == entry.entry_offset)
+                if raw_v5_entry and raw_v5_entry.entry_type == 'DW_RLE_base_addressx':
+                    self._emitline(base_template_indexed % (
+                        entry.entry_offset,
+                        raw_v5_entry.index,
+                        entry.base_address))
+                else:
+                    self._emitline(base_template % (
+                        entry.entry_offset if ver5 else first.entry_offset,
+                        entry.base_address))
             else:
                 raise NotImplementedError("Unknown object in a range list")
         last = range_list[-1]
