@@ -72,10 +72,16 @@ def run_test_on_file(filename, verbose=False, opt=None):
     else:
         options = [opt]
 
-    # TODO(sevaa): excluding two files from the --debug-dump=Ranges test until the maintainers
-    # of GNU binutils fix https://sourceware.org/bugzilla/show_bug.cgi?id=30781
-    if filename.endswith('dwarf_test_versions_mix.elf') or filename.endswith('dwarf_v5ops.so.elf'):
-        options.remove('--debug-dump=Ranges')
+    if filename.endswith('dwarf_debug_types.elf'):
+        # TODO: excluding the binary with .debug_types section until the length
+        #  the calculation for FDEs in binutils bug #31973 is fixed
+        #  https://sourceware.org/bugzilla/show_bug.cgi?id=31973
+        options.remove('--debug-dump=frames')
+        options.remove('--debug-dump=frames-interp')
+        # TODO: Stream parsing errors out with the .debug_aranges section
+        #  generated by IAR EWARM 9.20.4.
+        # https://github.com/STMicroelectronics/STM32CubeWB/issues/109
+        options.remove('--debug-dump=aranges')
 
     for option in options:
         if verbose: testlog.info("..option='%s'" % option)
@@ -89,6 +95,15 @@ def run_test_on_file(filename, verbose=False, opt=None):
                 testlog.info('.......................SKIPPED')
             continue
 
+        # TODO(sevaa): excluding the binary with CIE ahead of FDE until binutils' bug #31975 is fixed
+        if "dwarf_v4cie" in filename and option == "--debug-dump=frames-interp":
+            continue
+
+        # TODO(sevaa): excluding the binary with unaligned aranges entries. Readelf tried to recover
+        # but produces nonsensical output, but ultimately it's a toolchain bug (in IAR I presume).
+        if "dwarf_v4cie" in filename and option == "--debug-dump=aranges":
+            continue        
+
         # sevaa says: there is another shorted out test; in dwarf_lineprogramv5.elf, the two bytes at 0x2072 were
         # patched from 0x07 0x10 to 00 00.
         # Those represented the second instruction in the first FDE in .eh_frame. This changed the instruction
