@@ -9,10 +9,10 @@
 # This code is in the public domain
 #-------------------------------------------------------------------------------
 import os
+from io import BytesIO
 import sys
 import unittest
 
-from elftools.common.py3compat import BytesIO
 from elftools.elf.elffile import ELFFile
 from elftools.elf.relocation import RelocationHandler
 
